@@ -24,14 +24,22 @@ import json
 import locale
 import traceback
 import sys
+import queue
+from typing import NamedTuple, Optional
 
 from .version import ELECTRUM_VERSION
 from . import constants
 from .i18n import _
-from .util import make_aiohttp_session
+from .util import make_aiohttp_session, error_text_str_to_safe_str
 from .logging import describe_os_version, Logger, get_git_version
 
 
+class CrashReportResponse(NamedTuple):
+    status: Optional[str]
+    text: str
+    url: Optional[str]
+
+
 class BaseCrashReporter(Logger):
     report_server = "https://crashhub.electrum.org"
     config_key = "show_crash_reporter"
@@ -62,18 +70,36 @@ class BaseCrashReporter(Logger):
         Logger.__init__(self)
         self.exc_args = (exctype, value, tb)
 
-    def send_report(self, asyncio_loop, proxy, endpoint="/crash", *, timeout=None):
+    def send_report(self, asyncio_loop, proxy, *, timeout=None) -> CrashReportResponse:
+        # FIXME the caller needs to catch generic "Exception", as this method does not have a well-defined API...
         if constants.net.GENESIS[-4:] not in ["4943", "e26f"] and ".electrum.org" in BaseCrashReporter.report_server:
             # Gah! Some kind of altcoin wants to send us crash reports.
             raise Exception(_("Missing report URL."))
         report = self.get_traceback_info()
         report.update(self.get_additional_info())
         report = json.dumps(report)
-        coro = self.do_post(proxy, BaseCrashReporter.report_server + endpoint, data=report)
+        coro = self.do_post(proxy, BaseCrashReporter.report_server + "/crash.json", data=report)
         response = asyncio.run_coroutine_threadsafe(coro, asyncio_loop).result(timeout)
-        return response
+        self.logger.info(
+            f"Crash report sent. Got response [DO NOT TRUST THIS MESSAGE]: {error_text_str_to_safe_str(response)}")
+        response = json.loads(response)
+        assert isinstance(response, dict), type(response)
+        # sanitize URL
+        if location := response.get("location"):
+            assert isinstance(location, str)
+            base_issues_url = constants.GIT_REPO_ISSUES_URL
+            if not base_issues_url.endswith("/"):
+                base_issues_url = base_issues_url + "/"
+            if not location.startswith(base_issues_url):
+                location = None
+        ret = CrashReportResponse(
+            status=response.get("status"),
+            url=location,
+            text=_("Thanks for reporting this issue!"),
+        )
+        return ret
 
-    async def do_post(self, proxy, url, data):
+    async def do_post(self, proxy, url, data) -> str:
         async with make_aiohttp_session(proxy) as session:
             async with session.post(url, data=data, raise_for_status=True) as resp:
                 return await resp.text()
@@ -83,8 +109,8 @@ class BaseCrashReporter(Logger):
         stack = traceback.extract_tb(self.exc_args[2])
         readable_trace = self.__get_traceback_str_to_send()
         id = {
-            "file": stack[-1].filename,
-            "name": stack[-1].name,
+            "file": stack[-1].filename if len(stack) else '<no stack>',
+            "name": stack[-1].name if len(stack) else '<no stack>',
             "type": self.exc_args[0].__name__
         }
         return {
@@ -104,7 +130,7 @@ class BaseCrashReporter(Logger):
         }
         try:
             args["wallet_type"] = self.get_wallet_type()
-        except:
+        except Exception:
             # Maybe the wallet isn't loaded yet
             pass
         return args
@@ -131,6 +157,44 @@ class BaseCrashReporter(Logger):
         raise NotImplementedError
 
 
+class EarlyExceptionsQueue:
+    """Helper singleton for explicitly sending exceptions to crash reporter.
+
+    Typically the GUIs set up an "exception hook" that catches all otherwise
+    uncaught exceptions (which unroll the stack of a thread completely).
+    This class provides methods to report *any* exception, and queueing logic
+    that delays processing until the exception hook is set up.
+    """
+
+    _is_exc_hook_ready = False
+    _exc_queue = queue.Queue()
+
+    @classmethod
+    def set_hook_as_ready(cls):
+        """Flush the queue and disable it for future exceptions."""
+        if cls._is_exc_hook_ready:
+            return
+        cls._is_exc_hook_ready = True
+        while cls._exc_queue.qsize() > 0:
+            e = cls._exc_queue.get()
+            cls._send_exception_to_crash_reporter(e)
+
+    @classmethod
+    def send_exception_to_crash_reporter(cls, e: BaseException):
+        if cls._is_exc_hook_ready:
+            cls._send_exception_to_crash_reporter(e)
+        else:
+            cls._exc_queue.put(e)
+
+    @staticmethod
+    def _send_exception_to_crash_reporter(e: BaseException):
+        assert EarlyExceptionsQueue._is_exc_hook_ready
+        sys.excepthook(type(e), e, e.__traceback__)
+
+
+send_exception_to_crash_reporter = EarlyExceptionsQueue.send_exception_to_crash_reporter
+
+
 def trigger_crash():
     # note: do not change the type of the exception, the message,
     # or the name of this method. All reports generated through this
