@@ -24,7 +24,7 @@
 # SOFTWARE.
 from collections import defaultdict
 from math import floor, log10
-from typing import NamedTuple, List, Callable, Sequence, Union, Dict, Tuple
+from typing import NamedTuple, List, Callable, Sequence, Union, Dict, Tuple, Mapping, Type
 from decimal import Decimal
 
 from .bitcoin import sha256, COIN, is_address
@@ -410,7 +410,11 @@ class CoinChooserRandom(CoinChooserBase):
 
         for bkts_choose_from in bucket_sets:
             try:
-                def sfunds(bkts, *, bucket_value_sum):
+                def sfunds(
+                    bkts, *, bucket_value_sum,
+                    already_selected_buckets_value_sum=already_selected_buckets_value_sum,
+                    already_selected_buckets=already_selected_buckets,
+                ):
                     bucket_value_sum += already_selected_buckets_value_sum
                     return sufficient_funds(already_selected_buckets + bkts,
                                             bucket_value_sum=bucket_value_sum)
@@ -478,15 +482,15 @@ class CoinChooserPrivacy(CoinChooserRandom):
 
 COIN_CHOOSERS = {
     'Privacy': CoinChooserPrivacy,
-}
+}  # type: Mapping[str, Type[CoinChooserBase]]
 
 def get_name(config):
     kind = config.get('coin_chooser')
-    if not kind in COIN_CHOOSERS:
+    if kind not in COIN_CHOOSERS:
         kind = 'Privacy'
     return kind
 
-def get_coin_chooser(config):
+def get_coin_chooser(config) -> CoinChooserBase:
     klass = COIN_CHOOSERS[get_name(config)]
     # note: we enable enable_output_value_rounding by default as
     #       - for sacrificing a few satoshis
