@@ -17,7 +17,7 @@ from . import util
 from .bitcoin import COIN
 from .i18n import _
 from .util import (ThreadJob, make_dir, log_exceptions, OldTaskGroup,
-                   make_aiohttp_session, resource_path, EventListener, event_listener)
+                   make_aiohttp_session, resource_path, EventListener, event_listener, to_decimal)
 from .network import Network
 from .simple_config import SimpleConfig
 from .logging import Logger
@@ -34,19 +34,10 @@ CCY_PRECISIONS = {'BHD': 3, 'BIF': 0, 'BYR': 0, 'CLF': 4, 'CLP': 0,
                   'JOD': 3, 'JPY': 0, 'KMF': 0, 'KRW': 0, 'KWD': 3,
                   'LYD': 3, 'MGA': 1, 'MRO': 1, 'OMR': 3, 'PYG': 0,
                   'RWF': 0, 'TND': 3, 'UGX': 0, 'UYI': 0, 'VND': 0,
-                  'VUV': 0, 'XAF': 0, 'XAU': 4, 'XOF': 0, 'XPF': 0}
-
-
-def to_decimal(x: Union[str, float, int, Decimal]) -> Decimal:
-    # helper function mainly for float->Decimal conversion, i.e.:
-    #   >>> Decimal(41754.681)
-    #   Decimal('41754.680999999996856786310672760009765625')
-    #   >>> Decimal("41754.681")
-    #   Decimal('41754.681')
-    if isinstance(x, Decimal):
-        return x
-    return Decimal(str(x))
-
+                  'VUV': 0, 'XAF': 0, 'XAU': 4, 'XOF': 0, 'XPF': 0,
+                  # Cryptocurrencies
+                  'BTC': 8, 'LTC': 8, 'XRP': 6, 'ETH': 18,
+                  }
 
 POLL_PERIOD_SPOT_RATE = 150  # approx. every 2.5 minutes, try to refresh spot price
 EXPIRY_SPOT_RATE = 600       # spot price becomes stale after 10 minutes
@@ -113,7 +104,7 @@ class ExchangeBase(Logger):
         try:
             with open(filename, 'r', encoding='utf-8') as f:
                 h = json.loads(f.read())
-        except:
+        except Exception:
             return None
         if not h:  # e.g. empty dict
             return None
@@ -158,8 +149,13 @@ class ExchangeBase(Logger):
         return []
 
     def historical_rate(self, ccy: str, d_t: datetime) -> Decimal:
-        rate = self._history.get(ccy, {}).get(d_t.strftime('%Y-%m-%d')) or 'NaN'
-        return Decimal(rate)
+        date_str = d_t.strftime('%Y-%m-%d')
+        rate = self._history.get(ccy, {}).get(date_str) or 'NaN'
+        try:
+            return Decimal(rate)
+        except Exception:  # guard against garbage coming from exchange
+            #self.logger.debug(f"found corrupted historical_rate: {rate=!r}. for {ccy=} at {date_str}")
+            return Decimal('NaN')
 
     async def request_history(self, ccy: str) -> Dict[str, Union[str, float]]:
         raise NotImplementedError()  # implemented by subclasses
@@ -173,6 +169,8 @@ class ExchangeBase(Logger):
 
     def get_cached_spot_quote(self, ccy: str) -> Decimal:
         """Returns the cached exchange rate as a Decimal"""
+        if ccy == 'BTC':
+            return Decimal(1)
         rate = self._quotes.get(ccy)
         if rate is None:
             return Decimal('NaN')
@@ -389,14 +387,6 @@ class Kraken(ExchangeBase):
                      for k, v in json['result'].items())
 
 
-class LocalBitcoins(ExchangeBase):
-
-    async def get_rates(self, ccy):
-        json = await self.get_json('localbitcoins.com',
-                             '/bitcoinaverage/ticker-all-currencies/')
-        return dict([(r, to_decimal(json[r]['rates']['last'])) for r in json])
-
-
 class MercadoBitcoin(ExchangeBase):
 
     async def get_rates(self, ccy):
@@ -471,7 +461,7 @@ def get_exchanges_and_currencies():
     try:
         with open(path, 'r', encoding='utf-8') as f:
             return json.loads(f.read())
-    except:
+    except Exception:
         pass
     # or if not present, generate it now.
     print("cannot find currencies.json. will regenerate it now.")
@@ -485,7 +475,7 @@ def get_exchanges_and_currencies():
         try:
             d[name] = await exchange.get_currencies()
             print(name, "ok")
-        except:
+        except Exception:
             print(name, "error")
 
     async def query_all_exchanges_for_their_ccys_over_network():
@@ -521,10 +511,9 @@ def get_exchanges_by_ccy(history=True):
 
 class FxThread(ThreadJob, EventListener):
 
-    def __init__(self, config: SimpleConfig, network: Optional[Network]):
+    def __init__(self, *, config: SimpleConfig):
         ThreadJob.__init__(self)
         self.config = config
-        self.network = network
         self.register_callbacks()
         self.ccy = self.get_currency()
         self.history_used_spot = False
@@ -551,17 +540,27 @@ class FxThread(ThreadJob, EventListener):
         return d.get(ccy, [])
 
     @staticmethod
-    def remove_thousands_separator(text):
-        return text.replace(',', '') # FIXME use THOUSAND_SEPARATOR in util
+    def remove_thousands_separator(text: str) -> str:
+        return text.replace(util.THOUSANDS_SEP, "")
 
-    def ccy_amount_str(self, amount, commas):
-        prec = CCY_PRECISIONS.get(self.ccy, 2)
-        fmt_str = "{:%s.%df}" % ("," if commas else "", max(0, prec)) # FIXME use util.THOUSAND_SEPARATOR and util.DECIMAL_POINT
+    def ccy_amount_str(self, amount, *, add_thousands_sep: bool = False, ccy=None) -> str:
+        prec = CCY_PRECISIONS.get(self.ccy if ccy is None else ccy, 2)
+        fmt_str = "{:%s.%df}" % ("," if add_thousands_sep else "", max(0, prec))
         try:
             rounded_amount = round(amount, prec)
         except decimal.InvalidOperation:
             rounded_amount = amount
-        return fmt_str.format(rounded_amount)
+        text = fmt_str.format(rounded_amount)
+        # replace "," -> THOUSANDS_SEP
+        # replace "." -> DECIMAL_POINT
+        dp_loc = text.find(".")
+        text = text.replace(",", util.THOUSANDS_SEP)
+        if dp_loc == -1:
+            return text
+        return text[:dp_loc] + util.DECIMAL_POINT + text[dp_loc+1:]
+
+    def ccy_precision(self, ccy=None) -> int:
+        return CCY_PRECISIONS.get(self.ccy if ccy is None else ccy, 2)
 
     async def run(self):
         while True:
@@ -571,7 +570,7 @@ class FxThread(ThreadJob, EventListener):
                     await self._trigger.wait()
                     self._trigger.clear()
                 # we were manually triggered, so get historical rates
-                if self.is_enabled() and self.show_history():
+                if self.is_enabled() and self.has_history():
                     self.exchange.get_historical_rates(self.ccy, self.cache_dir)
             except TaskTimeout:
                 pass
@@ -585,26 +584,11 @@ class FxThread(ThreadJob, EventListener):
         self.config.set_key('use_exchange_rate', bool(b))
         self.trigger_update()
 
-    def get_history_config(self, *, allow_none=False):
-        val = self.config.get('history_rates', None)
-        if val is None and allow_none:
-            return None
-        return bool(val)
+    def can_have_history(self):
+        return self.is_enabled() and self.ccy in self.exchange.history_ccys()
 
-    def set_history_config(self, b):
-        self.config.set_key('history_rates', bool(b))
-
-    def get_history_capital_gains_config(self):
-        return bool(self.config.get('history_rates_capital_gains', False))
-
-    def set_history_capital_gains_config(self, b):
-        self.config.set_key('history_rates_capital_gains', bool(b))
-
-    def get_fiat_address_config(self):
-        return bool(self.config.get('fiat_address'))
-
-    def set_fiat_address_config(self, b):
-        self.config.set_key('fiat_address', bool(b))
+    def has_history(self) -> bool:
+        return self.can_have_history() and bool(self.config.get('history_rates', False))
 
     def get_currency(self) -> str:
         '''Use when dynamic fetching is needed'''
@@ -613,9 +597,6 @@ class FxThread(ThreadJob, EventListener):
     def config_exchange(self):
         return self.config.get('use_exchange', DEFAULT_EXCHANGE)
 
-    def show_history(self):
-        return self.is_enabled() and self.get_history_config() and self.ccy in self.exchange.history_ccys()
-
     def set_currency(self, ccy: str):
         self.ccy = ccy
         self.config.set_key('currency', ccy, True)
@@ -623,8 +604,8 @@ class FxThread(ThreadJob, EventListener):
         self.on_quotes()
 
     def trigger_update(self):
-        if self.network:
-            self.network.asyncio_loop.call_soon_threadsafe(self._trigger.set)
+        loop = util.get_asyncio_loop()
+        loop.call_soon_threadsafe(self._trigger.set)
 
     def set_exchange(self, name):
         class_ = globals().get(name) or globals().get(DEFAULT_EXCHANGE)
@@ -666,8 +647,11 @@ class FxThread(ThreadJob, EventListener):
 
     def get_fiat_status_text(self, btc_balance, base_unit, decimal_point):
         rate = self.exchange_rate()
-        return _("  (No FX rate available)") if rate.is_nan() else " 1 %s~%s %s" % (base_unit,
-            self.value_str(COIN / (10**(8 - decimal_point)), rate), self.ccy)
+        if rate.is_nan():
+            return _("  (No FX rate available)")
+        amount = 1000 if decimal_point == 0 else 1
+        value = self.value_str(amount * COIN / (10**(8 - decimal_point)), rate)
+        return " %d %s~%s %s" % (amount, base_unit, value, self.ccy)
 
     def fiat_value(self, satoshis, rate) -> Decimal:
         return Decimal('NaN') if satoshis is None else Decimal(satoshis) / COIN * Decimal(rate)
@@ -678,7 +662,7 @@ class FxThread(ThreadJob, EventListener):
     def format_fiat(self, value: Decimal) -> str:
         if value.is_nan():
             return _("No data")
-        return "%s" % (self.ccy_amount_str(value, True))
+        return self.ccy_amount_str(value, add_thousands_sep=True)
 
     def history_rate(self, d_t: Optional[datetime]) -> Decimal:
         if d_t is None:
