@@ -1,14 +1,16 @@
-from kivy.utils import get_color_from_hex, platform
+from kivy.utils import get_color_from_hex
+
+from electrum.gui.default_lang import get_default_language as _get_default_language
 
 
 def address_colors(wallet, addr):
     """
-    Chooses the appropriate text color and background color to 
+    Chooses the appropriate text color and background color to
     mark receiving, change and billing addresses.
 
     Returns: color, background_color
     """
-    
+
     # modified colors (textcolor, background_color) from electrum/gui/qt/util.py
     GREEN = ("#000000", "#8af296")
     YELLOW = ("#000000", "#ffff00")
@@ -24,13 +26,4 @@ def address_colors(wallet, addr):
 
 
 def get_default_language() -> str:
-    if platform != 'android':
-        return 'en_UK'
-    # FIXME: CJK/Arabic/etc languages do not work at all with kivy due to font issues,
-    #        so it is easiest to just default to English... (see #2032)
-    return 'en_UK'
-    # # try getting the language of the Android OS
-    # from jnius import autoclass
-    # Locale = autoclass("java.util.Locale")
-    # lang = str(Locale.getDefault().toString())
-    # return lang if lang else 'en_UK'
+    return _get_default_language(gui_name="kivy")
