@@ -23,30 +23,19 @@
 # CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
-import platform
-
 from PyQt5.QtCore import Qt
-from PyQt5.QtGui import *
-from PyQt5.QtWidgets import QHBoxLayout, QVBoxLayout, QLabel, QWidget
+from PyQt5.QtWidgets import QHBoxLayout, QWidget
 
 from .qrcodewidget import QRCodeWidget
+
 from electrum.i18n import _
 
-if platform.system() == 'Windows':
-    MONOSPACE_FONT = 'Lucida Console'
-elif platform.system() == 'Darwin':
-    MONOSPACE_FONT = 'Monaco'
-else:
-    MONOSPACE_FONT = 'monospace'
-
-column_index = 4
-
 
 class QR_Window(QWidget):
 
     def __init__(self, win):
         QWidget.__init__(self)
-        self.win = win
+        self.main_window = win
         self.setWindowTitle('Electrum - '+_('Payment Request'))
         self.setMinimumSize(800, 800)
         self.setFocusPolicy(Qt.NoFocus)
@@ -54,3 +43,6 @@ class QR_Window(QWidget):
         self.qrw = QRCodeWidget()
         main_box.addWidget(self.qrw, 1)
         self.setLayout(main_box)
+
+    def closeEvent(self, event):
+        self.main_window.receive_tab.qr_menu_action.setChecked(False)
