@@ -23,35 +23,56 @@
 # CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
+import enum
+
 from PyQt5.QtGui import QStandardItemModel, QStandardItem
 from PyQt5.QtCore import Qt
 from PyQt5.QtWidgets import (QDialog, QVBoxLayout, QPushButton, QLabel)
 
 from electrum.i18n import _
-from .util import MyTreeView, Buttons
+from .util import Buttons
+from .my_treeview import MyTreeView
 
 
 class WatcherList(MyTreeView):
-    def __init__(self, parent):
-        super().__init__(parent, self.create_menu, stretch_column=0)
+
+    class Columns(MyTreeView.BaseColumnsEnum):
+        OUTPOINT = enum.auto()
+        TX_COUNT = enum.auto()
+        STATUS = enum.auto()
+
+    headers = {
+        Columns.OUTPOINT: _('Outpoint'),
+        Columns.TX_COUNT: _('Tx'),
+        Columns.STATUS: _('Status'),
+    }
+
+    def __init__(self, parent: 'WatchtowerDialog'):
+        super().__init__(
+            parent=parent,
+            stretch_column=self.Columns.OUTPOINT,
+        )
+        self.parent = parent
         self.setModel(QStandardItemModel(self))
         self.setSortingEnabled(True)
         self.update()
 
-    def create_menu(self, x):
-        pass
-
     def update(self):
         if self.parent.lnwatcher is None:
             return
         self.model().clear()
-        self.update_headers({0:_('Outpoint'), 1:_('Tx'), 2:_('Status')})
+        self.update_headers(self.__class__.headers)
         lnwatcher = self.parent.lnwatcher
         l = lnwatcher.list_sweep_tx()
         for outpoint in l:
             n = lnwatcher.get_num_tx(outpoint)
             status = lnwatcher.get_channel_status(outpoint)
-            items = [QStandardItem(e) for e in [outpoint, "%d"%n, status]]
+            labels = [""] * len(self.Columns)
+            labels[self.Columns.OUTPOINT] = outpoint
+            labels[self.Columns.TX_COUNT] = str(n)
+            labels[self.Columns.STATUS] = status
+            items = [QStandardItem(e) for e in labels]
+            self.set_editability(items)
             self.model().insertRow(self.model().rowCount(), items)
         size = lnwatcher.sweepstore.filesize()
         self.parent.size_label.setText('Database size: %.2f Mb'%(size/1024/1024.))
@@ -67,7 +88,7 @@ class WatchtowerDialog(QDialog):
         assert self.network
         self.lnwatcher = self.network.local_watchtower
         self.setWindowTitle(_('Watchtower'))
-        self.setMinimumSize(600, 20)
+        self.setMinimumSize(600, 200)
         self.size_label = QLabel()
         self.watcher_list = WatcherList(self)
 
