@@ -84,18 +84,11 @@ class Plugin(ColdcardPlugin, QtPluginBase):
 
 
 class Coldcard_Handler(QtHandlerBase):
+    MESSAGE_DIALOG_TITLE = _("Coldcard Status")
 
     def __init__(self, win):
         super(Coldcard_Handler, self).__init__(win, 'Coldcard')
 
-    def message_dialog(self, msg):
-        self.clear_dialog()
-        self.dialog = dialog = WindowModalDialog(self.top_level_window(), _("Coldcard Status"))
-        l = QLabel(msg)
-        vbox = QVBoxLayout(dialog)
-        vbox.addWidget(l)
-        dialog.show()
-
 
 class CKCCSettingsDialog(WindowModalDialog):
 
@@ -233,7 +226,7 @@ class CKCCSettingsDialog(WindowModalDialog):
             self.window.show_error("Does not appear to be a Coldcard firmware file.\n\n%s" % exc)
             return
 
-        # TODO: 
+        # TODO:
         # - detect if they are trying to downgrade; aint gonna work
         # - warn them about the reboot?
         # - length checks
