@@ -66,13 +66,14 @@ class HW_PluginBase(BasePlugin):
         return self.parent.device_manager
 
     def create_device_from_hid_enumeration(self, d: dict, *, product_key) -> Optional['Device']:
+        # note: id_ needs to be unique between simultaneously connected devices,
+        #       and ideally unchanged while a device is connected.
         # Older versions of hid don't provide interface_number
         interface_number = d.get('interface_number', -1)
         usage_page = d['usage_page']
-        id_ = d['serial_number']
-        if len(id_) == 0:
-            id_ = str(d['path'])
-        id_ += str(interface_number) + str(usage_page)
+        # id_=str(d['path']) in itself might be sufficient, but this had to be touched
+        # a number of times already, so let's just go for the overkill approach:
+        id_ = f"{d['path']},{d['serial_number']},{interface_number},{usage_page}"
         device = Device(path=d['path'],
                         interface_number=interface_number,
                         id_=id_,
@@ -85,7 +86,7 @@ class HW_PluginBase(BasePlugin):
     def close_wallet(self, wallet: 'Abstract_Wallet'):
         for keystore in wallet.get_keystores():
             if isinstance(keystore, self.keystore_class):
-                self.device_manager().unpair_xpub(keystore.xpub)
+                self.device_manager().unpair_pairing_code(keystore.pairing_code())
                 if keystore.thread:
                     keystore.thread.stop()
 
@@ -156,7 +157,11 @@ class HW_PluginBase(BasePlugin):
                     or versiontuple(library_version) < self.minimum_library
                     or versiontuple(library_version) >= self.maximum_library):
                 raise LibraryFoundButUnusable(library_version=library_version)
-        except ImportError:
+        except ImportError as e:
+            self.libraries_available_message = (
+                _("Missing libraries for {}.").format(self.name)
+                + f"\n    {e!r}"
+            )
             return False
         except LibraryFoundButUnusable as e:
             library_version = e.library_version
@@ -243,8 +248,8 @@ class HardwareClientBase:
         during USB device enumeration (called for each unpaired device).
         Stored in the wallet file.
         """
-        # This functionality is optional. If not implemented just return None:
-        return None
+        root_fp = self.request_root_fingerprint_from_device()
+        return root_fp
 
     def has_usable_connection_with_device(self) -> bool:
         raise NotImplementedError()
@@ -349,25 +354,6 @@ def validate_op_return_output(output: TxOutput, *, max_size: int = None) -> None
         raise UserFacingException(_("Amount for OP_RETURN output must be zero."))
 
 
-def get_xpubs_and_der_suffixes_from_txinout(tx: PartialTransaction,
-                                            txinout: Union[PartialTxInput, PartialTxOutput]) \
-        -> List[Tuple[str, List[int]]]:
-    xfp_to_xpub_map = {xfp: bip32node for bip32node, (xfp, path)
-                       in tx.xpubs.items()}  # type: Dict[bytes, BIP32Node]
-    xfps = [txinout.bip32_paths[pubkey][0] for pubkey in txinout.pubkeys]
-    try:
-        xpubs = [xfp_to_xpub_map[xfp] for xfp in xfps]
-    except KeyError as e:
-        raise Exception(f"Partial transaction is missing global xpub for "
-                        f"fingerprint ({str(e)}) in input/output") from e
-    xpubs_and_deriv_suffixes = []
-    for bip32node, pubkey in zip(xpubs, txinout.pubkeys):
-        xfp, path = txinout.bip32_paths[pubkey]
-        der_suffix = list(path)[bip32node.depth:]
-        xpubs_and_deriv_suffixes.append((bip32node.to_xpub(), der_suffix))
-    return xpubs_and_deriv_suffixes
-
-
 def only_hook_if_libraries_available(func):
     # note: this decorator must wrap @hook, not the other way around,
     # as 'hook' uses the name of the function it wraps
